# Übung 11: Graph-Datenbank Neo4j

## Lernziele

-   Kennenlernen von Graph-Datenbanken am Beispiel Neo4j

## Vorbereitung

Die Aufgaben dieser Übung können mit einer Online oder Offline Version
von Neo4j durchführen.

##### Online-Console

Die [Neo4j-Sandbox](https://neo4j.com/sandbox-v2/) eignet sich gut für
diese Übung. Neben der Möglichkeit die nachfolgenden Aufgaben zu lösen
finden Sie weitere interessante Übungsbeispiele um Neo4j näher kennen zu
lernen.

Wählen Sie für die nachfolgenden Aufgaben eine neue, leere Sandbox.

##### Docker

Haben Sie Docker schon unter Verwendung und wollen auch Offline die
Übungen lösen können, eignet sich der [offizielle Docker
Container](https://hub.docker.com/_/neo4j/) von Neo4j. Einen
vorbereiteten Container `db2-neo4j` für die nachfolgenden Aufgaben ist
in [`crane.yml`](Helpers/crane.yml#L172) definiert.

Verbinden Sie sich nach erfolgreichem Start mit dem WebGUI der
[db2-neo4j Instanz](http://localhost:7474).

##### Microsoft Windows

1.  [Download Seite](http://neo4j.com/download/) im Browser öffnen
2.  Knopf "Download Community Edition" im Kästchen "For Individuals"
    anklicken, und danach "Other OS Downloads" wählen um die passende
    Datei für den Download auszuwählen.
3.  Die heruntergeladene Datei ausführen und dem Setup-Assistenten
    folgen:
    1.  Bei der Lizenz (GNU GPLv3), "I accept the agreement" wählen
    2.  Auf der letzten Seite "Run Neo4j Community Edition" ankreuzen,
        damit die Software nach Abschluss der Installation gleich
        gestartet wird
4.  Im Launcher-Fenster den "Start"-Knopf anklicken
5.  Sobald das System bereit ist, wechselt die Farbe des
    "Status"-Bereichs des Launcher-Fensters auf grün. Über den nun
    sichtbaren Link, kann auf die Weboberfläche der Graph-Datenbank
    zugegriffen werden.

##### Mac OSX

1.  [Download Seite](http://neo4j.com/download/) im Browser öffnen
2.  Knopf "Download Community Edition" im Kästchen "For Individuals"
    anklicken, und danach "Other OS Downloads" wählen um die passende
    Datei für den Download auszuwählen.
3.  Die heruntergeladene dmg-Datei installieren
4.  Im Launcher-Fenster den "Start"-Knopf anklicken
5.  Sobald das System bereit ist, wechselt die Farbe des
    "Status"-Bereichs des Launcher-Fensters auf grün. Über den nun
    sichtbaren Link, kann auf die Weboberfläche der Graph-Datenbank
    zugegriffen werden.

##### Linux

Folgen Sie der
[Anleitung](http://info.neo4j.com/download-thanks.html?edition=community&release=3.3.5&flavour=unix),
(benötigt Java 8). [Debian/Ubuntu-Pakete](http://debian.neo4j.org/) sind
auch erhältlich, benötigen aber ebenfalls Java 8.

### Zum DBMS verbinden

Das Web-Interface fragt nach Benutzername und Passwort - sofern Sie dies
nicht explizit ausgeschaltet haben (bspw. bei Verwendung des Docker
Container). Geben Sie beim ersten Mal den Standard-Benutzername und das
Standard-Passwort (beides `neo4j`) ein, und klicken sie "Connect". Sie
werden aufgefordert, ein neues Passwort zu wählen. (Bei erneutem
Verbinden verwenden Sie den Standard-Benutzername und ihr neu gewähltes
Passwort.) Schliessen Sie das `:play start`- und das
`:server connect`-Kästchen mit dem `x` oben rechts am jeweiligen
Kästchen.

### Daten laden

1.  Geben Sie in der Befehls-Zeile oben `:play movie-graph` ein
2.  Drücken Sie die Enter-Taste (oder klicken Sie den "play"-Knopf
    rechts)
3.  Blättern Sie durch Klick auf "\>" zur "Create"-Seite des Tutorials
4.  Klicken Sie in den Code, um ihn automatisch in die Befehls-Zeile zu
    kopieren
5.  Schliessen Sie das `:play movie-graph`-Kästchen mit dem "x" oben
    rechts an diesem Kästchen
6.  Drücken Sie Ctrl-Enter (oder klicken Sie den "play"-Knopf rechts).

    **Hinweise:**
    -   Ctrl-Enter wird wegen der mehrzeiligen Eingabe benötigt.

Wenn alles funktioniert hat, sehen Sie in einem neuen Kästchen einen
Ausschnitt der Daten. (Schauspieler Tom Hanks, Filme in denen er
gespielt hat, und deren Regisseure.)

**Hinweise:**

-   Kurze Einführung in
    [Cypher](https://neo4j.com/developer/cypher-query-language), die
    Abfragesprache von Neo4j.
-   [Cypher Reference
    Card](http://neo4j.com/docs/cypher-refcard/current/)
-   Zur [Analyse eines
    Query](http://neo4j.com/docs/developer-manual/current/cypher/query-tuning/how-do-i-profile-a-query/)
    können Sie `EXPLAIN` oder `PROFILE` vor das Query schreiben.

## Aufgaben

### Aufgabe 1: Knoten finden

##### 1a: Matchen über Attribut-Werte

``` neo4j
MATCH (tom {name: "Tom Hanks"}) RETURN tom
```

findet den Schauspieler mit Namen (Attribut `name`) "Tom Hanks". (Ein
Knoten im Graph.)

Film-Knoten unserer Daten haben den Namen des Films im Attribut `title`.

-   Finden Sie den Film mit dem Namen "Cloud Atlas".

    ***Lösungsvorschlag***

    ``` neo4j
    MATCH (f {title: "Cloud Atlas"}) RETURN f
    ```

##### 1b: Begrenzen der Ausgabe

Wie bei `SELECT` in SQL kann die Ausgabe von `RETURN` durch Anhängen von
weiteren Klauseln modifiziert (z.B. sortiert mit `ORDER BY`) werden.

``` neo4j
MATCH (people:Person) RETURN people.name
```

gibt die Namen aller 133 Personen in der Datenbank aus. Wenn Sie ein
Vielfaches dieser Zahl erhalten, haben Sie die Daten ziemlich sicher
mehrfach importiert.

-   Geben Sie die Namen von 10 Personen aus (egal welche).

    ***Lösungsvorschlag***

    ``` neo4j
    MATCH (people:Person) RETURN people.name LIMIT 10
    ```

**Tipp:**

-   Die Klausel hierfür heisst gleich wie in SQL.

##### 1c: Matchen über beliebige Bedingungen

Die erste Abfrage hätten wir auch als

``` neo4j
MATCH (tom) WHERE tom.name = "Tom Hanks" RETURN tom
```

oder

``` neo4j
MATCH (tom:Person) WHERE tom.name = "Tom Hanks" RETURN tom
```

schreiben können. `:Person` ist dabei ein Label auf den gematchten
Knoten. `tom` ist ein Name für die gematchten Knoten, der in den
weiteren Klauseln der Abfrage (hier `WHERE` und `RETURN`) zur Verfügung
steht.

-   Finden Sie alle Filme, die in den 90er-Jahren (1990 bis und
    mit 1999) veröffentlicht wurden, und geben Sie deren Titel aus. (Das
    Veröffentlichungs-Jahr ist in Attribut released.)

    ***Lösungsvorschlag***

    ``` neo4j
    MATCH (movies:Movie)
    WHERE movies.released >= 1990 AND movies.released <= 1999
    RETURN movies.title
    ```

### Aufgabe 2: Knoten anhand ihrer Beziehungen Abfragen

Der MATCH-Klausel gibt man ein Muster (pattern) für die gewünschte
Ausgabe mit. Da die bisherigen Muster sich auf einzelne Knoten bezogen,
war das vielleicht nicht gleich erkennbar. Kommen Beziehungen mit ins
Spiel, wird es offensichtlicher.

Knoten werden im Muster durch Ausdrücke in runden Klammern `()`
dargestellt, Beziehungen durch Ausdrücke in eckigen Klammern `[]`.
Verbunden werden sie über Minus-Zeichen `-` und wenn die Richtung der
Beziehung eingeschränkt werden soll, ergänzt man diese mit kleiner `<`
oder grösser `>` zu einem Pfeil `<-[...]-`.

In den Ausdrücken innerhalb der Knoten und Beziehungen kann man den
gematchten Elementen Namen vergeben (oben bereits gesehen) oder sie
aufgrund ihrer Labels einschränken (ebenfalls in der Aufgabe oben
bereits vorgekommen). In geschweiften Klammern `{}` innerhalb der
Ausdrücke können die Attribut-Werte eingeschränkt werden.

1.  Erklären Sie die Bedeutung der Abfrage

    ``` neo4j
    MATCH (tom:Person {name: "Tom Hanks"})-[:ACTED_IN]->(tomHanksMovies)
    RETURN tom,tomHanksMovies
    ```

    ***Lösungsvorschlag***

    Listet alle Filme auf in denen "Tom Hanks" als Schauspieler
    mitgespielt hat.

2.  Fragen Sie ab, wer bei "Cloud Atlas" Regie geführt hat und geben Sie
    die Namen dieser Personen aus. Entsprechende Beziehungen richten
    sich vom Regisseur/der Regisseurin zum Film und tragen das Label
    `:DIRECTED`.

    ***Lösungsvorschlag***

    ``` neo4j
    MATCH (ca:Movie {title: "Cloud Atlas"})<-[:DIRECTED]-(regs:Person)
    RETURN ca.title,regs.name
    ```

3.  Fragen Sie ab, welche Schauspieler zusammen mit Tom Hanks in Filmen
    gespielt haben.

    ***Lösungsvorschlag***

    ``` neo4j
    MATCH (tom:Person {name: "Tom Hanks"})-[:ACTED_IN]->(thmovies)
        <-[:ACTED_IN]-(others:Person)
    RETURN thmovies.title, others.name
    ```

4.  Erklären Sie die Bedeutung der Abfrage

    ``` neo4j
    MATCH (people:Person)-[relatedTo]-(:Movie {title: "Cloud Atlas"})
    RETURN people.name, Type(relatedTo), relatedTo
    ```

    ***Lösungsvorschlag***

    Welche Personen haben am Film "Cloud Atlas" mitgewirkt und in
    welcher Rolle.

### Aufgabe 3: Pfade im Graph

Im Gegensatz zu relationalen Datenbanken können Graph-Datenbanken auch
Muster mit langen Pfaden oder mit Pfaden von variabler Länge effizient
mit den Daten abgleichen.

Auch die Eingabe solcher Abfragen ist einfach:

``` neo4j
MATCH (bacon:Person {name:"Kevin Bacon"})-[*1..4]-(hollywood)
RETURN DISTINCT hollywood
```

So werden alle Knoten gefunden, die über bis zu 4 Beziehungs-Hüpfer
(Pfade durch den Graph mit Länge 1 bis 4) etwas mit **Kevin Bacon** zu
tun haben.

-   Was denken Sie, was

    ``` neo4j
    MATCH p=shortestPath(
        (bacon:Person {name:"Kevin Bacon"})-[*]-(meg:Person {name:"Meg Ryan"})
    )
    RETURN p
    ```

    abfragt?

    $`\Rightarrow`$ Experimentieren Sie mit verschiedenen Abfragen.

    ***Lösungsvorschlag***

    Ermittelt die kürzeste Verbindung zwischen "Kevin Bacon" und "Meg
    Ryan".

### Aufgabe 4: Freundes-Freunde und andere Bekannte

Wir wollen Tom Hanks Vorschläge machen, mit wem zusammen er weitere
Filme drehen könnte. Dazu betrachten wir Schauspieler und
Schauspielerinnen, die mit seinen mit-Schauspielern Filme gedreht haben,
in denen Hanks selbst aber nicht mitgespielt hat. Je besser eine dieser
Personen in Hollywood vernetzt ist, um so höher gewichten wir die
jeweilige Empfehlung.

``` neo4j
MATCH (tom:Person {name:"Tom Hanks"})-[:ACTED_IN]->(m)<-[:ACTED_IN]-(coActors),
(coActors)-[:ACTED_IN]->(m2)<-[:ACTED_IN]-(cocoActors)
WHERE NOT (tom)-[:ACTED_IN]->(m2)
RETURN cocoActors.name AS Recommended, count(*) AS Strength
ORDER BY Strength DESC
```

Tom Cruise wäre wohl ein guter Kandidat.

-   Wer könnte die beiden bekannt machen? Am besten jemand, der mit
    beiden schon gedreht hat. Schreiben Sie eine Abfrage, die solche
    Personen findet.

    ***Lösungsvorschlag***

    ``` neo4j
    MATCH (tom:Person {name:"Tom Hanks"})-[:ACTED_IN]->(m)<-[:ACTED_IN]-(coActors),
      (coActors)-[:ACTED_IN]->(m2)<-[:ACTED_IN]-(cruise:Person {name:"Tom Cruise"})
    RETURN tom, m, coActors, m2, cruise
    ```

### Aufgabe 5: Daten löschen

Löschen Sie alle Filme und Personen in der Datenbank, sowie deren
Beziehungen:

``` neo4j
MATCH (a:Person),(m:Movie) OPTIONAL MATCH (a)-[r1]-(), (m)-[r2]-() DELETE
a,r1,m,r2
```

***Alternative zum Löschen aller Nodes und Relationen***

``` neo4j
MATCH (n)
DETACH DELETE n
```

-   Überprüfen Sie, dass die Datenbank nun leer ist:

    ``` neo4j
    MATCH (n) RETURN n
    ```

## Musterlösung

Diese Übung basiert auf dem Movie Graph-Tutorial, mit dessen ersten
Schritt wir die Daten geladen haben. Alle Queries und knappe Erklärungen
derselben finden Sie daher im Tutorial wenn Sie weiterblättern anstatt
das Kästchen zu schliessen. Sie können das Tutorial erneut starten, in
dem sie

``` neo4j
:play movie-graph
```

in der Befehls-Zeile der Neo4j-Weboberfläche eingeben.

**Hinweis:**

-   Falls Sie die initial eingelesenen Daten nicht wieder gelöscht
    haben, überspringen Sie den entsprechenden Schritt, da die Daten
    sonst mehrfach in der Datenbank sind.
-   Mit Klick auf das Stecknadel-Symbol oben rechts am Tutorial-Kästchen
    können Sie das Kästchen "anheften", so dass es von weiteren Ausgaben
    nicht nach unten weggeschoben wird.

Die Musterlösung zu dieser Übung finden Sie auf [dieser
Seite](GraphDB/README.solutions.md) oder als
[PDF](/../-/jobs/artifacts/master/raw/GraphDB-Solutions.pdf?job=Replication)
Datei.

---
keywords:
- database
- graph
- neo4j
title: 'Übung 11: Graph-Datenbank Neo4j'
---

